/////////////////////////////////////////////////////////////////////////////////////////////
//
//		ARENA C     $Revision$
//
/////////////////////////////////////////////////////////////////////////////////////////////

/*  This file was forked from ParseBSA files from Stanza's WinArena open source project  */

/*  By GASNOT Marc ( arkem )
 *
 *  $Date$
 */

/*  Revision History
 *
 *  $Log$
 */



/////////////////////////////////////////////////////////////////////////////////////////////
//  PREPROCESSOR
/////////////////////////////////////////////////////////////////////////////////////////////

/*  Misc  */
#define   ARENA_DATA_MAX   8



/////////////////////////////////////////////////////////////////////////////////////////////
//  HEADERS
/////////////////////////////////////////////////////////////////////////////////////////////

/*  Standard  */
#include   <errno.h>
#include   <libio.h>
#include   <stdio.h>
#include   <stdlib.h>
#include   <string.h>
#include   <sys/types.h>
//#include   <unistd.h>
//#include   <time.h>
//#include   <math.h>

/*  Libraries  */

/*  Program  */
//#include   "winarena.h"
#include   "arena.h"
#include   "arenatab.h"



/////////////////////////////////////////////////////////////////////////////////////////////
//  DEFINES
/////////////////////////////////////////////////////////////////////////////////////////////

/*  Files formats  */
// Animations
#define   ARENA_ANIM_CFA   0
#define   ARENA_ANIM_DFA   1
#define   ARENA_ANIM_FLC   2
#define   ARENA_ANIM_RCI   3
// Images
#define   ARENA_IMAGE_CIF   0
#define   ARENA_IMAGE_DAT   2
#define   ARENA_IMAGE_FOG   3
#define   ARENA_IMAGE_IMG   4
#define   ARENA_IMAGE_SET   5
//enum { ARENA_CFA=0, ARENA_CIF=1, ARENA_DFA=2, ARENA_IMG=3, ARENA_INF=4, ARENA_MIF=5, ARENA_SET=6, ARENA_VOC=7, ARENA_XFM=8, ARENA_XMI=9 };

/*  IMG file Header flags  */
#define   ARENA_FLAG_COMPRLE   0x0002   // Compressed in RLE
#define   ARENA_FLAG_COMPUN1   0x0004   // Compressed unknow 1
#define   ARENA_FLAG_COMPUN2   0x0008   // Compressed unknow 2 - Record start with uncompressed size (2 bytes word)
#define   ARENA_FLAG_PALETTE   0x0100   // Palette - Last 256x3 bytes contain a custom palette
#define   ARENA_FLAG_TRNSCOL   0x0800   // Transparency colors used by image



/////////////////////////////////////////////////////////////////////////////////////////////
//  STRUCTURES
/////////////////////////////////////////////////////////////////////////////////////////////

/*  GLOBAL.BSA  */
// FILE index element
//struct arena_bsaindex
//{
//	char Name[12];	// 8.3 formatted file name, without any '\0' terminator
//	int16_t U1;		// Unkown, possibly unused
//	int16_t Size;
//	int16_t U2;		// Unkown, possibly unused
//};
// MEMORIZED index element
struct arena_bsaindex
{
	char        name[16];   // Name
	int         offset;     // Byte offset from start of BSA file
	int         size;       // Size of resource, in bytes
};

/*  GFX file Headers  */
// CFA
//struct cfahead
//{
//	int16_t WidthUncompressed;   // actual width of the image
//	int16_t Height;
//	int16_t WidthCompressed;     // bytes/line of bit-packed image
//	int16_t u2;
//	int16_t u3;
//	char BitsPerPixel;           // indicates how to demux the encoded pixels
//	char FrameCount;
//	int16_t HeaderSize;          // this also includes the color LUT following the header
//};
// DFA files are used to static animations, such as fire bits, bartenders,
// jugglers, and other decorative sprites.  These are typically a static
// frame with a small block of animation.
//struct dfahead
//{
//	int16_t FrameCount;
//	int16_t u2;
//	int16_t u3;
//	int16_t Width;
//	int16_t Height;
//	int16_t CompressedSize;
//};
// IMG & CIF
//struct arena_imghead
//{
//	int16_t     offsetX;
//	int16_t     offsetY;
//	int16_t     width;
//	int16_t     height;
//	u_int16_t   flags;
//	int16_t     recordSize;
//};   // A two bytes word follows containing size of image when uncompressed, only present if ARENA_FLAG_COMPUN2 flag is set
// MNU seems to be an IMG file without the 'recordSize' word at the end of the header
//struct arena_mnuhead
//{
//	int16_t     offsetX;      // Unused
//	int16_t     offsetY;      // Unused
//	int16_t     width;        // Always 320
//	int16_t     height;       // Always 200
//	u_int16_t   flags;        // Always 0x0904: Compressed1 + paletted + 0x0800 unknown (see ARENA_IMG_ flags)
//};

/*  Sound file Headers  */
// Header block found at the beginning of each sound file.  These are Creative
// Voice files, but there does not appear to be any information about the
// audio formatting.  All audio is 8-bit mono.  I have no idea whether these
// are linear, A-law, or mu-law encoded.  The format bits that look like they
// should indicate this are not set, so I assume they're linear.  Mixing them
// by linear addition doesn't seem to change the value, so I assume linear,
// but I'm an expert on video, not audio.
//struct vochead
//{
//	char  CreativeVoice[19];
//	char  MarkEOF;
//	int16_t  DataOffset;
//	int16_t  Version;				// 0x010A
//	int16_t  TwosCompVersion;		// 0x1129
//};



/////////////////////////////////////////////////////////////////////////////////////////////
//  GLOBAL VARIABLES
/////////////////////////////////////////////////////////////////////////////////////////////

/*  Misc  */
// Auto-Init
static int                 arena_init = 0;
// Data structures list
static struct arena_data   *arena_dataList[ARENA_DATA_MAX];
// Shared file pointer
static FILE                *arena_tempFile;

/*  Global.bsa  */
// File path if found
static char                    *arena_globalPath   = NULL;
// Content index
static struct arena_bsaindex   *arena_globalIndex  = NULL;
// Files number
int                            arena_globalNumber  = 0;   // Used externaly
// File names list
static char                    **arena_globalNames = NULL;



/////////////////////////////////////////////////////////////////////////////////////////////
//  FONCTIONS DECLARATIONS
/////////////////////////////////////////////////////////////////////////////////////////////

/*  Files Read  */
// Read an Animation file
static struct arena_data* arena_ReadAnim( char*, int );
// Read a DAT file
static struct arena_data* arena_ReadDAT( char* );
// Read a Font DAT file
static struct arena_data* arena_ReadFont( char*, int, int );
// Read an Image file
static struct arena_data* arena_ReadImage( char*, int );
// Read a lightmap file
static struct arena_data* arena_ReadLightMap( char* );
// Read a Palette file
static struct arena_data* arena_ReadPalette( char* );
// Read a Text file
static struct arena_data* arena_ReadText( char* );

/*  Files Write  */
// Write a DAT file
static int arena_WriteDAT( struct arena_data*, char* );
// Write an Image file
static int arena_WriteImage( struct arena_data*, char* );
// Write a Text file
static int arena_WriteText( struct arena_data*, char* );

/*  Utilities  */
// Open a file stream in specified directory or in GLOBAL.BSA if possible
static FILE *arena_OpenFile( char*, int* );
// Decode or recode a INF text data
static void arena_ProcessINF( char*, int );
// Decode RLE compressed image data
static int arena_DecodeRLE( u_int8_t*, u_int8_t*, int );   // Source buffer, destination buffer, uncompressed image size
// Extract file name frome the file path
static char* arena_ExtractFileName( char* );

/*  Management  */
// Allocate a new data structure
static struct arena_data* arena_NewData();
// Data list initialisation
static int arena_Init( char* );



/////////////////////////////////////////////////////////////////////////////////////////////
//  READ FONCTIONS DESCRIPTIONS
/////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////
//  arena_GlobalList  //
////////////////////////
// Get GLOBAL.BSA files list
char** arena_GlobalList( char *globalPath )
{
	/*  Local Variables  */
	int   i;

	/*  Check Init  */
	if( arena_init == 0 )
		arena_Init( globalPath );

	/*  Check GLOBAL.BSA  */
	if( arena_globalPath == NULL ) {
		fputs( "arena_GlobalList couldn't create file list: GLOBAL.BSA was not found", stderr );
		return( NULL );
	};

	/*  Allocate names list  */
	if( arena_globalNames == NULL ) {
		arena_globalNames = (char**)malloc( sizeof(char**) * arena_globalNumber );
		if( arena_globalNames == NULL ) {
			fprintf( stderr, "arena_GlobalList couldn't allocate file list: %s", strerror(errno) );
			return( NULL );
		};
	};

	/*  Fill names list  */
	i = 0;
	while( i < arena_globalNumber ) {
		arena_globalNames[i] = arena_globalIndex[i].name;
//		printf( "%s\n", arena_globalNames[i] );   // Test
		++i;
	};

	/*  End  */
	return( arena_globalNames );
};


///////////////////////////
//  arena_GetGlobalFile  //
///////////////////////////
// Extract a raw GLOBAL.BSA file
struct arena_data* arena_GetGlobalFile( char *globalPath, char *name )
{
	/*  Local Variables  */
	struct arena_data   *data;
	FILE                *file;
	int                 index;

	/*  Check Init  */
	if( arena_init == 0 )
		arena_Init( globalPath );

	/*  Check GLOBAL.BSA  */
	if( arena_globalPath == NULL ) {
		fputs( "arena_GetGlobalFile couldn't access data: GLOBAL.BSA was not found", stderr );
		return( NULL );
	};

	/*  Search file in Index  */
	index = 0;
	while( index < arena_globalNumber ) {
		if( strcmp(arena_globalIndex[index].name,name) == 0 )
			break;
		else
			++index;
	};
	if( index >= arena_globalNumber ) {
		fprintf( stderr, "arena_GetGlobalFile couldn't find %s in GLOBAL.BSA\n", name );
		return( NULL );
	};

	/*  Allocate data  */
	// Structure
	data = arena_NewData();
	if( data == NULL )
		return( NULL );
	// File
	data->length = arena_globalIndex[index].size;
	data->data = malloc( data->length );
	if( data->data == NULL ) {
		fprintf( stderr, "arena_GetGlobalFile couldn't allocate %s data: %s\n", name, strerror(errno) );
		arena_FreeData( data );
		return( NULL );
	};

	/*  Load file  */
	// Open
	file = fopen( globalPath, "rb" );
	if( file == NULL ) {
		fprintf( stderr, "arena_GetGlobalFile couldn't open GLOBAL.BSA: %s\n", strerror(errno) );
		arena_FreeData( data );
		return( NULL );
	};
	// Seek to data
	if( fseek(file,arena_globalIndex[index].offset,SEEK_SET) != 0 ) {
		fprintf( stderr, "arena_GetGlobalFile couldn't seek to %s in GLOBAL.BSA: %s\n", name, strerror(errno) );
		fclose( file );
		arena_FreeData( data );
		return( NULL );
	};
	// Read file
	if( fread(data->data,1,data->length,file) < data->length ) {
		fprintf( stderr, "arena_GetGlobalFile: Error reading %s in GLOBAL.BSA: %s\n", name, strerror(errno) );
		fclose( file );
		arena_FreeData( data );
		return( NULL );
	};

	/*  End  */
	fclose( file );
//	printf( "%s Extracted from %s \n", name, globalPath );   // Test
	return( data );
};


//////////////////////
//  arena_ReadFile  //
//////////////////////
// Read an Arena format file
struct arena_data* arena_ReadFile( char *filePath )
{
	/*  Local Variables  */
	struct arena_data   *data = NULL;
	char                *ext;

	/*  Check Init  */
	if( arena_init == 0 )
		arena_Init( filePath );

	/*  Check extension  */
	ext = &filePath[strlen(filePath)-3];
	if( strcasecmp("CDE",ext) == 0 )   // Unknown
		fputs( "CDE format is not yet supported ...!", stderr );
//		data = arena_ReadUnknown( filePath );
	else if( strcasecmp("CEL",ext) == 0 )   // Unknown
		fputs( "CEL format is not yet supported ...!", stderr );
//		data = arena_ReadUnknown( filePath );
	else if( strcasecmp("CFA",ext) == 0 )   // Animation ?
		fputs( "CFA format is not yet supported ...!", stderr );
//		data = arena_ReadAnim( filePath, ARENA_ANIM_CFA );
	else if( strcasecmp("CIF",ext) == 0 )   // Multiple images or character weapons anim
//		fputs( "CIF format is not yet supported ...!", stderr );
		data = arena_ReadImage( filePath, ARENA_IMAGE_CIF );
	else if( strcasecmp("CLR",ext) == 0 )   // Savegame names
		data = arena_ReadText( filePath );
	else if( strcasecmp("COL",ext) == 0 )   // Colors palette
		data = arena_ReadPalette( filePath );
	else if( strcasecmp("DAT",ext) == 0 )   // Misc fonts, text, images or unknown
		data = arena_ReadDAT( filePath );
	else if( strcasecmp("DFA",ext) == 0 )   // Single image with a small block of animation
		fputs( "DFA format is not yet supported ...!", stderr );
//		data = arena_ReadAnim( filePath, ARENA_ANIM_DFA );
	else if( strcasecmp("FLC",ext) == 0 )   // FLC Animation
		fputs( "FLC format is not yet supported ...!", stderr );
//		data = arena_ReadAnim( filePath, ARENA_ANIM_FLC );
	else if( strcasecmp("IMG",ext) == 0 )   // Single image
		data = arena_ReadImage( filePath, ARENA_IMAGE_IMG );
	else if( strcasecmp("INF",ext) == 0 )   // Text formated map infos
		data = arena_ReadText( filePath );
	else if( strcasecmp("LGT",ext) == 0 )   // LightMap
		data = arena_ReadLightMap( filePath );
	else if( strcasecmp("LST",ext) == 0 )   // Mixed ASCII and binary spells list
		fputs( "LST format is not yet supported ...!", stderr );
//		data = arena_ReadList( filePath );
	else if( strcasecmp("MIF",ext) == 0 )   // Map
		fputs( "MIF format is not yet supported ...!", stderr );
//		data = arena_ReadMap( filePath );
	else if( strcasecmp("MNU",ext) == 0 )   // 320 * 200 IMG with a header: 0x0004 (compressed), 0x0100 (paletted) and 0x0800 (unknown), but without RecordSize word
		fputs( "MNU format is not yet supported ...!", stderr );
//		data = arena_ReadMap?( filePath );
	else if( strcasecmp("NTR",ext) == 0 )   // CITYINTR: City intro texts
		data = arena_ReadText( filePath );
	else if( strcasecmp("RCI",ext) == 0 )   // 320x100 x 5 frames Animation
		data = arena_ReadAnim( filePath, ARENA_ANIM_RCI );
	else if( strcasecmp("RMD",ext) == 0 )   // Exterior map clusters ?
		fputs( "RMD format is not yet supported ...!", stderr );
//		data = arena_ReadUnknown( filePath );
	else if( strcasecmp("SET",ext) == 0 )   // Multiple images
		data = arena_ReadImage( filePath, ARENA_IMAGE_SET );
	else if( strcasecmp("TXT",ext) == 0 ) {   // Misc text or image ?
		if( strcasecmp(arena_ExtractFileName(filePath), "FOG.TXT") == 0 )
			data = arena_ReadImage( filePath, ARENA_IMAGE_FOG );
		else
			data = arena_ReadText( filePath );
	}
	else if( strcasecmp("VOC",ext) == 0 )   // VOC Sound
		fputs( "VOC format is not yet supported ...!", stderr );
//		data = arena_ReadSound( filePath );
	else if( strcasecmp("XFM",ext) == 0 )   // FM Music
		fputs( "XFM format is not yet supported ...!", stderr );
//		data = arena_ReadMusic( filePath );
	else if( strcasecmp("XMI",ext) == 0 )   // MIDI Music
		fputs( "XMI format is not yet supported ...!", stderr );
//		data = arena_ReadMusic( filePath );
	else
		fprintf( stderr, "Unknown %s format isn't supported ...!\n", ext );

	/*  End  */
//	printf( "File %s loaded\n", filePath );   // Test
	return( data );
};


//////////////////////
//  arena_ReadAnim  //
//////////////////////
// Read an Animation file
struct arena_data* arena_ReadAnim( char *filePath, int type )
{
	/*  Local Variables  */
	struct arena_data   *data = NULL;
	FILE                *file;
	int                 size = 0, noHeader = 0;

	/*  Allocate data structure  */
	data = arena_NewData();
	if( data == NULL )
		return( NULL );
	data->type = ARENA_ANIM;
	
	/*  Open image  */
	file = arena_OpenFile( filePath, &size );
	if( file == NULL ) {
		arena_FreeData( data );
		return( NULL );
	};

	/*  RCI Files  */
	if( type == ARENA_ANIM_RCI ) {
		fseek( file, 0, SEEK_END );
		size = ftell( file );
		rewind( file );
		if( size != 160000 ) {
			fputs( "RCI file not supported", stderr );
			arena_FreeData( data );
			fclose( file );
			return( NULL );
		};
		printf( "Animation %s has no header...\n", filePath );   // Test
		data->length = 5;
		puts( "Images number: 5" );   // Test
		data->width  = 320;
		puts( "Width: 320" );   // Test
		data->height = 100;
		puts( "Height: 100" );   // Test
		noHeader = 1;
	};

	/*  Allocate animation data  */
	data->data = (u_int8_t*)malloc( size );
	if( data->data == NULL )   {
		fprintf( stderr, "arena_ReadAnim failed to allocate %s data: %s\n", filePath, strerror(errno) );
		arena_FreeData( data );
		fclose( file );
		return( NULL );
	};

	/*  Read animation  */
	if( fread(data->data,1,size,file) < size ) {
		fprintf( stderr, "arena_ReadAnim failed to read %s data: %s\n", filePath, strerror(errno) );
		arena_FreeData( data );
		fclose( file );
		return( NULL );
	};

	/*  End  */
	fclose( file );
	return( data );
};


/////////////////////
//  arena_ReadDAT  //
/////////////////////
// Read a DAT file
struct arena_data* arena_ReadDAT( char *filePath )
{
	/*  Local Variables  */
	struct arena_data   *data = NULL;
	char                *fileName = arena_ExtractFileName( filePath );
	int                 i;

	/*  Check filename  */
	i = 0;
	while( i < ARENATAB_DAT ) {
		if( strcasecmp(arenatab_datFiles[i].name,fileName) == 0 ) {
			switch( arenatab_datFiles[i].type ) {
				case ARENA_FONT:
					printf( "%s is a font file - Character height: %d\n", fileName, arenatab_datFiles[i].var2 );   // Test
					data = arena_ReadFont( filePath, arenatab_datFiles[i].var1, arenatab_datFiles[i].var2 );
					return( data );
				case ARENA_TEXT:
					printf( "%s is a DAT text file. Loading...\n", fileName );   // Test
					data = arena_ReadText( filePath );
					return( data );
				case ARENA_UBIN:
					printf( "%s seems to be an image file. Trying to load it...\n", fileName );   // Test
					data = arena_ReadImage( filePath, ARENA_IMAGE_DAT );
					return( data );
//					return( NULL );
			};
			break;
		}
		else
			++i;
	};

	/*  End  */
	fprintf( stderr, "%s is a totally unknown file ...!\n", fileName );
	return( NULL );
};


//////////////////////
//  arena_ReadFont  //
//////////////////////
// Read a Font DAT file
struct arena_data* arena_ReadFont( char *filePath, int length, int height )
{
	/*  Local Variables  */
	struct arena_data   *data;
	struct arena_char   *font;
	FILE                *file;
	int                 i, l, c, w;
	u_int16_t           mask;
	u_int8_t            b;

	/*  Allocate data structure  */
	data = arena_NewData();
	if( data == NULL )
		return( NULL );
	data->type   = ARENA_FONT;
	data->length = length;
	data->height = height;

	/*  Allocate (and clear) font data  */
	data->data = (struct arena_char*)calloc( 96, sizeof(struct arena_char) );
	if( data->data == NULL )   {
		fprintf( stderr, "arena_ReadFont failed to allocate %s data: %s\n", filePath, strerror(errno) );
		arena_FreeData( data );
		return( NULL );
	};
	font = (struct arena_char*)data->data;

	/*  Open font file  */
	file = fopen( filePath, "rb" );
	if( file == NULL ) {
		fprintf( stderr, "arena_ReadFont couldn't open file %s: %s\n", filePath, strerror(errno) );
		arena_FreeData( data );
		return( NULL );
	};

	/*  Read characters height values  */
	i = 1;
	while( i < 96 )	{
		if( fread(&b,1,1,file) < 1 )	{
			fprintf( stderr, "arena_ReadFont: Error reading character %d height value in %s ...!\n", i, filePath );
			arena_FreeData( data );
			return( NULL );
		};
		font[i].height = (int)b;
		++i;
	};

	/*  Read characters data  */
	i = 1;
	while( i < 96 )	{
		// Read character
		if( fread(font[i].lines,2,height,file) < height )	{
			fprintf( stderr, "arena_ReadFont: Error reading character %d in %s ...!\n", i, filePath );
			fclose( file );
			arena_FreeData( data );
			return( NULL );
		};
		// Scan for width
		w = 0;
		l = 0;
		while( l < height)   {
			mask = 0x8000;
			c = 0;
			while( c < 16 )   {
				if( (font[i].lines[l] & mask) != 0 )   {
					if (w < (c + 1))
						w = c + 1;
				};
				mask = mask >> 1;
				++c;
			};
			++l;
		};
		w += 1;   // Pad out the width an extra pixel for inter-character spacing
		font[i].width = w;
		++i;
	};

	/*  Initialize space character  */
	font[0].width  = 4;
	font[0].height = height + 1;

	/*  End  */
	fclose( file );
	return( data );
};


///////////////////////
//  arena_ReadImage  //
///////////////////////
// Read an Image file
struct arena_data* arena_ReadImage( char *filePath, int type )
{
	/*  Local Variables  */
	struct arena_data   *data = NULL;
	u_int8_t            palette[256][3];
	u_int8_t            *comp = NULL;
	FILE                *file;
	char                *fileName = arena_ExtractFileName( filePath );
	int                 i, size = 0, noHeader = 0, compressed = 0, paletted = 0;

	/*  Allocate data structure  */
	data = arena_NewData();
	if( data == NULL )
		return( NULL );
	data->type = ARENA_IMAGE;
	data->length = 1;   // Used by multiple images files
	data->offsetX = 0;
	data->offsetY = 0;
	
	/*  Open image  */
	file = arena_OpenFile( filePath, &size );
	if( file == NULL ) {
		arena_FreeData( data );
		return( NULL );
	};

	/*  Images with no header  */
	// Various sizes
	i = 0;
	while( i < ARENATAB_IMG_NOHEADER ) {
		if( strcmp(fileName,arenatab_imagesNoHeader[i].name) == 0 ) {
			printf( "Image %s has no header\n", fileName );   // Test
			data->width  = arenatab_imagesNoHeader[i].width;
			printf( "Width: %d\n", data->width );   // Test
			data->height = arenatab_imagesNoHeader[i].height;
			printf( "Height: %d\n", data->height );   // Test
			size = data->width * data->height;
			noHeader = 1;
		};
		++i;
	};
	if( noHeader == 0 ) {
		if( size == 0 ) {   // File is not in GLOBAL.BSA
			fseek( file, 0, SEEK_END );
			size = ftell( file );
			rewind( file );
		};
		if( (size==4096) && (strcasecmp(&fileName[strlen(fileName)-3],"IMG")==0) ) {   // 64 x 64 IMG files
			printf( "Image %s has no header...\n", fileName );   // Test
			data->width  = 64;
			puts( "Width: 64" );   // Test
			data->height = 64;
			puts( "Height: 64" );   // Test
			noHeader = 1;
		}
		else if( strcasecmp(&fileName[strlen(fileName)-3],"SET") == 0 ) {   // 64 x 64 x number SET files
			printf( "Image %s has no header...\n", fileName );   // Test
			data->length = size / 4096;
			printf( "Images number: %d\n", data->length );   // Test
			data->width  = 64;
			puts( "Width: 64" );   // Test
			data->height = 64;	// * data->length;
//			printf( "Height: %d\n", data->height );   // Test
			puts( "Height: 64" );   // Test
			noHeader = 1;
		};
	};

	/*  Images with header  */
	if( noHeader == 0 ) {
		// Offsets
		printf( "Image %s seems to have a header...\n", fileName );   // Test
		data->offsetX = 0;
		fread( (u_int16_t*)&data->offsetX, 2, 1, file );   // OffsetX
		printf( "X offset: %d\n", data->offsetX );   // Test
		data->offsetY = 0;
		fread( (u_int16_t*)&data->offsetY, 2, 1, file );   // OffsetY
		printf( "Y offset: %d\n", data->offsetY );   // Test
		// Dimensions
		data->width = 0;
		fread( (u_int16_t*)&data->width, 2, 1, file );     // Width
		printf( "Width: %d\n", data->width );   // Test
		data->height = 0;
		fread( (u_int16_t*)&data->height, 2, 1, file );    // Height
		printf( "Height: %d\n", data->height );   // Test
		// Flags
		i = 0;
		fread( (u_int16_t*)&i, 2, 1, file );               // Flags
		if( (i & ARENA_FLAG_COMPRLE) > 0 ) {
			puts( "Flag 0x0002 set (RLE compression)" );   // Test
			compressed = ARENA_FLAG_COMPRLE;
		};
		if( (i & ARENA_FLAG_COMPUN1) > 0 ) {
			puts( "Flag 0x0004 set (Unknown compression 1)" );   // Test
			compressed = ARENA_FLAG_COMPUN1;
		};
		if( (i & ARENA_FLAG_COMPUN2) > 0 ) {
			puts( "Flag 0x0008 set (Unknown compression 2)" );   // Test
			compressed = ARENA_FLAG_COMPUN2;
		};
		if( (i & ARENA_FLAG_PALETTE) > 0 ) {
			puts( "Flag 0x0100 set (Image is paletted)" );   // Test
			paletted = 1;
		};
		if( (i & ARENA_FLAG_TRNSCOL) > 0 )
			puts( "Flag 0x0800 set (Unknown)" );   // Test
		printf( "Flag field: 0x%X\n", (unsigned int)i );   // Test
		// Record size
		size = 0;
		fread( (u_int16_t*)&size, 2, 1, file );            // Record size
		printf( "Record size: %d\n", size );   // Test
		// Special case
		if( compressed == ARENA_FLAG_COMPUN2 ) {           // Unknown compression 2: Record begin with a 2 bytes word that contains uncompressed image size
			i = 0;
			fread( (u_int16_t*)&i, 2, 1, file );
			printf( "Start of record uncompressed size: %d\n", i );   // Test
			size -= 2;
		};
		// Really a header ?
		if( (data->width<=0) || (data->height<=0) || ((data->width*data->height)<0) || ((unsigned long long int)(data->width*data->height)>64000) ) {
			fprintf( stderr, "arena_ReadImage: Incorrect image dimensions in %s. File is unreadable.\n", fileName );
			arena_FreeData( data );
			fclose( file );
			return( NULL );
		};
	};

	/*  Allocate image data  */
	data->data = (u_int8_t*)malloc( data->width * data->height * data->length );   // size value is too small in case of compressed image
	if( data->data == NULL )   {
		fprintf( stderr, "arena_ReadImage failed to allocate %s data: %s\n", fileName, strerror(errno) );
		arena_FreeData( data );
		fclose( file );
		return( NULL );
	};

	/*  Read image  */
//	printf( "Image position: %d\n", (int)ftell(file) );   // Test
	if( compressed > 0 ) {   // Compressed
		// Allocate compressed data
		comp = (u_int8_t*)malloc( size );
		if( comp == NULL )   {
			fprintf( stderr, "arena_ReadImage failed to allocate %s compressed data: %s\n", fileName, strerror(errno) );
			arena_FreeData( data );
			fclose( file );
			return( NULL );
		};
		// Read compressed data
		if( fread(comp,1,size,file) < size ) {
			fprintf( stderr, "arena_ReadImage failed to read %s compressed data: %s\n", fileName, strerror(errno) );
			arena_FreeData( data );
			fclose( file );
			return( NULL );
		};
		// Unpack to data struct
		if( compressed == ARENA_FLAG_COMPRLE ) {
			puts( "Image uses RLE compression" );
			arena_DecodeRLE( comp, data->data, (data->width*data->height) );
		}
		else
			fprintf( stderr, "arena_ReadImage couldn't read %s data: All compressed images are not supported yet ...!\n", fileName );
		// Free compressed data
		free( comp );
	}
	else {   // Not compressed
		if( fread(data->data,1,size,file) < size ) {
			fprintf( stderr, "arena_ReadImage failed to read %s data: %s\n", fileName, strerror(errno) );
			arena_FreeData( data );
			fclose( file );
			return( NULL );
		};
	};

	/*  Palette  */
	if( paletted != 0 ) {
//		printf( "Palette position: %d\n", (int)ftell(file) );   // Test
		// Read
		if( fread(palette,3,256,file) < 256 ) {
			fprintf( stderr, "arena_ReadImage: Error reading palette in %s ...!\n", fileName );
			arena_FreeData( data );
			fclose( file );
			return( NULL );
		};
		// Allocate
		data->palette = (struct arena_color*)malloc( sizeof(struct arena_color)*256 );
		if( data->palette == NULL ) {
			fprintf( stderr, "arena_ReadImage failed to allocate %s palette data: %s\n", fileName, strerror(errno) );
			arena_FreeData( data );
			fclose( file );
			return( NULL );
		};
		// Extract
		i = 0;
		while( i < 256 ) {
			data->palette[i].red   = palette[i][0] << 2;
			data->palette[i].green = palette[i][1] << 2;
			data->palette[i].blue  = palette[i][2] << 2;
			++i;
		};
	};

	/*  End  */
//	printf( "End position: %d\n", (int)ftell(file) );   // Test
	fclose( file );
	return( data );
};


//////////////////////////
//  arena_ReadLightMap  //
//////////////////////////
// Read a lightmap file
struct arena_data* arena_ReadLightMap( char *filePath )
{
	/*  Local Variables  */
	struct arena_data   *data = NULL;
	FILE                *file;

	/*  Allocate data structure  */
	data = arena_NewData();
	if( data == NULL )
		return( NULL );
	data->type = ARENA_LIGHTMAP;

	/*  Allocate data  */
	data->data = (struct arena_color*)malloc( (13*256) );
	if( data->data == NULL ) {
		fprintf( stderr, "arena_ReadLightMap failed to allocate %s data: %s\n", filePath, strerror(errno) );
		arena_FreeData( data );
		return( NULL );
	};

	/*  Open palette file  */
	file = fopen( filePath, "rb" );
	if( file == NULL ) {
		fprintf( stderr, "arena_ReadLightMap couldn't open file %s: %s\n", filePath, strerror(errno) );
		arena_FreeData( data );
		return( NULL );
	};

	/*  Read LightMap  */
	if( fread(data->data,1,(13*256),file) < (13*256) ) {
		fprintf( stderr, "arena_ReadLightMap: Error reading data in %s ...!\n", filePath );
		arena_FreeData( data );
		fclose( file );
		return( NULL );
	};

	/*  End  */
	fclose( file );
	return( data );
};


/////////////////////////
//  arena_ReadPalette  //
/////////////////////////
// Read an Palette file
struct arena_data* arena_ReadPalette( char *filePath )
{
	/*  Local Variables  */
	struct arena_data   *data = NULL;
	u_int8_t            palette[256][3];
	FILE                *file;
	int                 i;

	/*  Allocate data structure  */
	data = arena_NewData();
	if( data == NULL )
		return( NULL );
	data->type = ARENA_PALETTE;

	/*  Allocate palette data  */
	data->palette = (struct arena_color*)malloc( sizeof(struct arena_color)*256 );
	if( data->palette == NULL ) {
		fprintf( stderr, "arena_ReadPalette failed to allocate %s palette data: %s\n", filePath, strerror(errno) );
		arena_FreeData( data );
		return( NULL );
	};

	/*  Open palette file  */
	file = fopen( filePath, "rb" );
	if( file == NULL ) {
		fprintf( stderr, "arena_ReadPalette couldn't open file %s: %s\n", filePath, strerror(errno) );
		arena_FreeData( data );
		return( NULL );
	};

	/*  Read palette file  */
	// Skip first 8 info bytes
	fseek( file, 8, SEEK_SET );
	// Read palette
	if( fread(palette,3,256,file) < 256 ) {
		fprintf( stderr, "arena_ReadPalette: Error reading data in %s ...!\n", filePath );
		arena_FreeData( data );
		fclose( file );
		return( NULL );
	};

	/*  Extract palette  */
	i = 0;
	while( i < 256 ) {
		data->palette[i].red   = palette[i][0];   // << 2;
		data->palette[i].green = palette[i][1];   // << 2;
		data->palette[i].blue  = palette[i][2];   // << 2;
		++i;
	};

	/*  End  */
	fclose( file );
	return( data );
};


//////////////////////
//  arena_ReadText  //
//////////////////////
// Read a Text file
struct arena_data* arena_ReadText( char *filePath )
{
	/*  Local Variables  */
	struct arena_data   *data = NULL;
	FILE                *file;
	char                *fileName = arena_ExtractFileName( filePath );
	int                 size = 0;

	/*  Allocate data structure  */
	data = arena_NewData();
	if( data == NULL )
		return( NULL );
	data->type = ARENA_TEXT;
	
	/*  Open file  */
	file = arena_OpenFile( filePath, &size );
	if( file == NULL ) {
		arena_FreeData( data );
		return( NULL );
	};
	
	/*  Obtain file size  */
	if( size == 0 ) {
		fseek( file, 0, SEEK_END );
		size = (int)ftell( file );
		rewind( file );
	};
	data->length = size;

	/*  Allocate data  */
	data->data = malloc( size );
	if( data->data == NULL ) {
		fprintf( stderr, "arena_ReadText: Error trying to allocate text buffer for %s: %s\n", fileName, strerror(errno) );
		arena_FreeData( data );
		fclose( file );
		return( NULL );
	};

	/*  Read data  */
	if( fread(data->data,1,size,file) < size ) {
		fprintf( stderr, "arena_ReadText: Error reading data in %s ...!\n", fileName );
		arena_FreeData( data );
		fclose( file );
		return( NULL );
	};

	/*  INF files decoding  */
	if(  strcasecmp("INF",&fileName[strlen(fileName)-3]) == 0 ) {
		if( *(char*)data->data != '@' )
			arena_ProcessINF( (char*)data->data, size );
		else
			printf( "File %s is already decoded\n", fileName );   // Info
	};

	/*  End  */
	fclose( file );
	return( data );
};



/////////////////////////////////////////////////////////////////////////////////////////////
//  WRITE FONCTIONS DESCRIPTIONS
/////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////
//  arena_WriteFile  //
///////////////////////
// Write an Arena format file
int arena_WriteFile( struct arena_data *data, char *filePath )
{
	/*  Local Variables  */
	char   *ext;

	/*  Check Init  */
	if( arena_init == 0 )
		arena_Init( filePath );

	/*  Check extension  */
	ext = &filePath[strlen(filePath)-3];
	if( strcasecmp("DAT",ext) == 0 ) {
		if( arena_WriteDAT(data,filePath) > 0 )
			return( 1 );
	}
	else if( strcasecmp("IMG",ext) == 0 ) {
		if( arena_WriteImage(data,filePath) > 0 )
			return( 1 );
	}
	else if( strcasecmp("INF",ext) == 0 ) {
		if( arena_WriteText(data,filePath) > 0 )
			return( 1 );
	}
	else
		fprintf( stderr, "%s format isn't supported ...!\n", ext );

	/*  End  */
	printf( "File %s has been saved\n", filePath );   // Test
	return( 0 );
};


//////////////////////
//  arena_WriteDAT  //
//////////////////////
// Write a DAT file
int arena_WriteDAT( struct arena_data *data, char *filePath )
{
	/*  Local Variables  */
	int   err;

	/*  Check file type  */
	switch( data->type ) {
		case ARENA_FONT:
			fprintf( stderr, "Couldn't save %s: Fonts save not supported.\n", filePath );
			err = 1;
			break;
		case ARENA_TEXT:
			err = arena_WriteText( data, filePath );
			break;
		case ARENA_UBIN:
			fprintf( stderr, "Couldn't save %s: Unknown file format.\n", filePath );
			err = 1;
			break;
		default:
			fprintf( stderr, "Couldn't save %s: Unknown file type.\n", filePath );
			err = 1;
	};

	/*  End  */
	return( err );
};


////////////////////////
//  arena_WriteImage  //
////////////////////////
// Write an Image file
int arena_WriteImage( struct arena_data *data, char *filePath )
{
	/*  Local Variables  */
	u_int16_t   flags;
	u_int8_t    palette[256][3];
	FILE        *file;
	int         i, noHeader = 0;

	/*  Verify that image exists  */   // !!! NEED WORK TO BE REMOVED !!!
	file = fopen( filePath, "rb" );
	if( file == NULL ) {
		fprintf( stderr, "Existing destination file needed to write: %s\n", strerror(errno) );
		return( 1 );
	};
	// Get flags and close
	fseek( file, 8, SEEK_SET );
	fread( &flags, 2, 1, file );
	fclose( file );

	/*  Open image in write mode  */
	file = fopen( filePath, "wb" );
	if( file == NULL ) {
		fprintf( stderr, "arena_WriteImage couldn't open file %s: %s\n", filePath, strerror(errno) );
		return( 1 );
	};

	/*  Images with no header  */
	// Variable size IMG
	i = 0;
	while( i < ARENATAB_IMG_NOHEADER ) {
		if( strcmp(filePath,arenatab_imagesNoHeader[i].name) == 0 ) {
			printf( "Image %s has no header\n", filePath );   // Test
			noHeader = 1;
			flags = 0;
		};
		++i;
	};
	if( noHeader == 0 ) {
		if( ((data->width*data->height)==4096) && (strcasecmp(&filePath[strlen(filePath)-3],"IMG")==0) ) {   // 64 x 64 IMG files
			printf( "Image %s has no header...\n", filePath );   // Test
			noHeader = 1;
			flags = 0;
		}
		else if( strcasecmp(&filePath[strlen(filePath)-3],"SET") == 0 ) {   // 64 x 64 x number SET files
			printf( "Image %s has no header...\n", filePath );   // Test
			noHeader = 1;
		};
	};

	/*  Write header  */
	if( noHeader == 0 ) {
		printf( "Image %s seems to have a header\n", filePath );   // Test
		fwrite( (u_int16_t*)&data->offsetX, 2, 1, file );   // Offsets X
		fwrite( (u_int16_t*)&data->offsetY, 2, 1, file );   // Offsets Y
		fwrite( (u_int16_t*)&data->width, 2, 1, file );     // Width
		fwrite( (u_int16_t*)&data->height, 2, 1, file );    // Height
		flags = flags & ( ARENA_FLAG_PALETTE + ARENA_FLAG_TRNSCOL );
		fwrite( &flags, 2, 1, file );                       // Flags (only palette and transparency bits are supported in write mode)
		i = data->width * data->height;
		fwrite( (u_int16_t*)&i, 2, 1, file );               // Record size (uncompressed in write mode)
		// Uncompressed size word is here when ARENA_FLAG_COMPUN2 flag (packer 2) is set. Not supported in write mode.
	};

	/*  Write image  */
//	if( fwrite(data->data,1,(data->width*data->height),file) < (data->width*data->height) ) {
	if( fwrite(data->data,1,(data->width*data->height*data->length),file) < (data->width*data->height*data->length) ) {
		fprintf( stderr, "arena_WriteImage failed to write data to %s: %s\n", filePath, strerror(errno) );
		fclose( file );
		return( 1 );
	};

	/*  Palette  */
	if( (flags & ARENA_FLAG_PALETTE) > 0 ) {
		// Generate
		i = 0;
		while( i < 256 ) {   // This fonction assumes that source colors uses 8 bits per component. The game uses 6.
			palette[i][0] = data->palette[i].red   >> 2;
			palette[i][1] = data->palette[i].green >> 2;
			palette[i][2] = data->palette[i].blue  >> 2;
			++i;
		};
		// Write
		if( fwrite(palette,3,256,file) < 256 ) {
			fprintf( stderr, "arena_WriteImage: Error writeing palette in %s ...!\n", filePath );
			fclose( file );
			return( 1 );
		};
	};

	/*  End  */
	fclose( file );
	return( 0 );
};


///////////////////////
//  arena_WriteText  //
///////////////////////
// Write an Text file
int arena_WriteText( struct arena_data *data, char *filePath )
{
	/*  Local Variables  */
	FILE        *file;

	/*  INF files recoding  */
	if(  strcasecmp("INF",&filePath[strlen(filePath)-3]) == 0 )
		puts( "INF files recoding hasn't been implemented yet (not really usefull). File is not encrypted but usable." );
//		arena_ProcessINF( (char*)data->data, data->length );

	/*  Open text in write mode  */
	file = fopen( filePath, "wb" );
	if( file == NULL ) {
		fprintf( stderr, "arena_WriteText couldn't open file %s: %s\n", filePath, strerror(errno) );
		return( 1 );
	};

	/*  Write text  */
	if( fwrite(data->data,1,data->length,file) < data->length ) {
		fprintf( stderr, "arena_WriteText failed to write data to %s: %s\n", filePath, strerror(errno) );
		fclose( file );
		return( 1 );
	};

	/*  End  */
	fclose( file );
	return( 0 );
};



/////////////////////////////////////////////////////////////////////////////////////////////
//  UTILITY FONCTIONS DESCRIPTIONS
/////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////
//  arena_OpenFile  //
//////////////////////
// Open a file stream in specified directory or in GLOBAL.BSA if possible
FILE *arena_OpenFile( char *filePath, int *size )
{
	/*  Local Variables  */
	char   *fileName = arena_ExtractFileName( filePath );
	int    index;

	/*  Try to open file normally  */
	arena_tempFile = fopen( filePath, "rb" );
	// No need to go further, and no GLOBAL.BSA = no size
	if( arena_tempFile != NULL )
		return( arena_tempFile );
//	fprintf( stderr, "Couldn't open %s: %s\n", filePath, strerror(errno) );   // Test

	/*  No GLOBAL.BSA, no need to continue  */
	if( arena_globalPath == NULL ) {
		fprintf( stderr, "GLOBAL.BSA not found. %s load cancelled ...!\n", strerror(errno) );
		return( NULL );
	};

	/*  Look into GLOBAL.BSA index  */
	index = 0;
	while( index < arena_globalNumber ) {
//		printf( "Index %d: \t %s\n", index, arena_globalIndex[index].name );
		if( strcmp(arena_globalIndex[index].name,fileName) == 0 ) {
//			printf( "%s found at index %d in GLOBAL.BSA\n", fileName, index );   // Test
			break;
		}
		else
			++index;
	};

	/*  Not found  */
	if( index >= arena_globalNumber ) {
		fprintf( stderr, "File %s wasn't found in specified directory, nor in GLOBAL.BSA ...!\n", fileName );   // Test
		return( NULL );
	};

	/*  Open GLOBAL.BSA  */
	arena_tempFile = fopen( arena_globalPath, "rb" );
	if( arena_tempFile == NULL ) {
		fprintf( stderr, "arena_OpenFile: Error trying to open GLOBAL.BSA: %s\n", strerror(errno) );
		return( NULL );
	};

	/*  Seek to data  */
	if( fseek(arena_tempFile,arena_globalIndex[index].offset,SEEK_SET) != 0 ) {
		fprintf( stderr, "arena_OpenFile: Error seeking to file %s in GLOBAL.BSA: %s\n", fileName, strerror(errno) );
		fclose( arena_tempFile );
		return( NULL );
	};

	/*  Success  */
	*size = arena_globalIndex[index].size;
	return( arena_tempFile );
};


////////////////////////
//  arena_ProcessINF  //
////////////////////////
// Decode a INF text data
void arena_ProcessINF( char *text, int size )
{
	/*  Local Variables  */
	int    rowShift[16] = { 0,  9, 7, 14, 3, 14, 5, 12, 1, 10, 7, 14, 4, 15, 6, 12 };
	int    idx, encoded, lobits, hibits, rownum, colnum, index, muxen;
	int    inverseCodeTableLow[16][16];
	int    x, y, trans;
	char   c;

//	puts( "Decoding INF file..." );   // Temp

	// Decoding the low four bits experimentally derived the foreward translation
	// table.  To perform the inverse translation, this function needs to be called
	// first to fill in the inverseCodeTableLow[][] table.
	//
	// Make inverseCodeTable
	y = 0;
	while( y < 16 ) {
		x = 0;
		while( x < 16 ) {
			// For readability purposes, the code tables were filled in with text.
			// So those hex digits need to be translated to their corresponding
			// 4-bit numerical values.
			//
			// Hex Char To Int
			c = arenatab_CodeTableLow[y][x];
			if ((c >= 'A') && (c <= 'F')) {
				trans = ( (char)c - 'A' + 10 );
			}
			else
				trans = ( (char)c - '0' );
			inverseCodeTableLow[y][trans] = x;
			++x;
		};
		++y;
	};

	// This function decodes the encoded text.  It relies upon an inverse translation
	// table, which is filled in by MakeInverseCodeTable(), so that function needs to
	// be called before attempting to decode any text.
	//
	// Separate table look-up operations are used on the low and high nibbles of each
	// byte.  The low 4 bits may be decoded by a simple table look-up, whereas the
	// high four bits require a translation of one array index.
	//
	// The obfuscation mechanism is cyclic, repeating every 16 bytes for the low
	// nibble, and every 256 bytes for the high nibble.  Therefore, the whole text
	// chunk should be decoded at one time so DeobfuscateINF() can derive the hashing
	// values from each byte's position in the text buffer.
	//
	// The code therefore decodes one byte at a time, splitting off the high and low
	// bits for separate processing.  Once translated, the bits are combined and
	// written back into the text buffer.
	//
	// Deobfuscate INF
	idx = 0;
	while( idx < size ) {
		encoded = (int)(text[idx]);
		lobits  = (encoded     ) & 0xF;
		hibits  = (encoded >> 4) & 0xF;
		rownum  = (idx >> 4) & 0xF;
		colnum  = (idx     ) & 0xF;
		// The low 4 bits are decoded by a simple table look-up operation.
		lobits = inverseCodeTableLow[colnum][lobits];
		// The high 4 bits are subjected to a rotation that repeats every
		// 16 bytes.  This requires using one table to compute an index
		// into the second table.
		index  = ( rowShift[colnum] + rownum ) & 0xF;
		// Using the massaged index, the actual table look-up can be performed
		// to decode the high 4 bits.
		c = arenatab_InverseCodeTableHigh[index][hibits];
		if ( (c >= 'A') && (c <= 'F') ) {
			hibits = (char)c - 'A' + 10;
		}
		else
			hibits = (char)c - '0';
		// Then the low and high bits are muxed together again and written
		// back into the text buffer.
		muxen = ( hibits << 4 ) | lobits;
		text[idx] = (char)muxen;
		++idx;
	};
};




///////////////////////
//  arena_DecodeRLE  //
///////////////////////
// Decode RLE compressed image data
int arena_DecodeRLE( u_int8_t *src, u_int8_t *dst, int size )
{
	/*  Local Variables  */
	u_int8_t   sample, value;
	int        count, j, o, i = 0;

	/*  Decode  */
	o = 0;
	while( o < size ) {
		sample = src[i++];
		// Compressed packet
		if (sample & 0x80) {
			value = src[i++];
			count = (int)sample - 0x7F;
			j = 0;
			while( j < count ) {
				dst[o++] = value;
				++j;
			};
		}
		else {
			count = (int)sample + 1;
			j = 0;
			while( j < count ) {
				dst[o++] = src[i++];
				++j;
			};
		};
	};

	/*  End  */
	return( i );
};


/////////////////////////////
//  arena_ExtractFileName  //
/////////////////////////////
// Extract file name frome the file path
char* arena_ExtractFileName( char *filePath )
{
	/*  Local Variables  */
	int   i;

	/*  Extract file name  */
	i = strlen( filePath );
	while( i > 0 )   {
		if( filePath[i] == '/' )
//		if( filePath[i] == '\' )   // Windows version
			return( &filePath[i+1] );
		--i;
	};

	/*  End  */
	return( filePath );   // File name isn't preceeded py a path name
};



/////////////////////////////////////////////////////////////////////////////////////////////
//  MANAGEMENT FONCTIONS DESCRIPTIONS
/////////////////////////////////////////////////////////////////////////////////////////////

//////////////////
//  arena_Init  //
//////////////////
// Data list initialisation
int arena_Init( char *filePath )
{
	/*  Local Variables  */
	FILE   *file;
	int    i, offset = 2;

	/*  Clear data list  */
	i = 0;
	while( i < ARENA_DATA_MAX )   {
		arena_dataList[i] = NULL;
		++i;
	};
	arena_init = 1;

	/*  Generate GLOBAL.BSA file path  */
	i = strlen( filePath ) - strlen( arena_ExtractFileName(filePath) );
	arena_globalPath = (char*)malloc( i + 11 );
	if( arena_globalPath == NULL ) {
		fprintf( stderr, "arena_Init couldn't allocate GLOBAL.BSA file path: %s\n\n", strerror(errno) );
		return( 1 );
	};
	strncpy( arena_globalPath, filePath, i );
	strcpy( &arena_globalPath[i], "GLOBAL.BSA" );
//	printf( "GLOBAL.BSA file path: %s\n", arena_globalPath );   // Test

	/*  Open GLOBAL.BSA  */
	file = fopen( arena_globalPath, "rb" );
	if( file == NULL ) {
//		fprintf( stderr, "arena_Init couldn't open file GLOBAL.BSA: %s\n\n", strerror(errno) );   // Test
		free( arena_globalPath );
		arena_globalPath = NULL;
		return( 1 );
	};

	/*  Read GLOBAL.BSA index position  */
	fread( (u_int16_t*)&arena_globalNumber, 2, 1, file );
	if( fseek(file,-(arena_globalNumber*18),SEEK_END) != 0 ) {
		fprintf( stderr, "arena_Init couldn't go to GLOBAL.BSA file index: %s\n\n", strerror(errno) );
		free( arena_globalPath );
		arena_globalPath = NULL;
		fclose( file );
		return( 1 );
	};
//	printf( "GLOBAL.BSA data space size: %d, Index size: %d, Index elements: %d \n", (int)(ftell(file)-2), (arena_globalNumber*18), arena_globalNumber );   // Test

	/*  Allocate GLOBAL.BSA index table  */
	arena_globalIndex = (struct arena_bsaindex*)malloc( arena_globalNumber * sizeof(struct arena_bsaindex) );
	if( arena_globalPath == NULL ) {
		fprintf( stderr, "arena_Init couldn't allocate GLOBAL.BSA file path: %s\n\n", strerror(errno) );
		free( arena_globalPath );
		arena_globalPath = NULL;
		fclose( file );
		return( 1 );
	};

	/*  Read GLOBAL.BSA index  */
	i = 0;
	while( i < arena_globalNumber ) {
		// Name
		if( fread(arena_globalIndex[i].name,1,14,file) < 14 ) {
			fileno( file );
			fprintf( stderr, "arena_Init: Error reading name of element %d in GLOBAL.BSA file index: %s\n\n", (i+1), strerror(errno) );
			free( arena_globalPath );
			arena_globalPath = NULL;
			free( arena_globalIndex );
			arena_globalIndex = NULL;
			fclose( file );
			return( 1 );
		};
		// File size
		arena_globalIndex[i].size = 0;
		if( fread((u_int16_t*)&arena_globalIndex[i].size,2,1,file) < 1 ) {
			fprintf( stderr, "arena_Init: Error reading size of element %d in GLOBAL.BSA file index: %s\n\n", (i+1), strerror(errno) );
			free( arena_globalPath );
			arena_globalPath = NULL;
			free( arena_globalIndex );
			arena_globalIndex = NULL;
			fclose( file );
			return( 1 );
		};
		// Unknown 2
		if( fseek(file,2,SEEK_CUR) != 0 ) {
			fprintf( stderr, "arena_Init: Error skipping unknown data 2 of element %d in GLOBAL.BSA file index: %s\n\n", (i+1), strerror(errno) );
			free( arena_globalPath );
			arena_globalPath = NULL;
			free( arena_globalIndex );
			arena_globalIndex = NULL;
			fclose( file );
			return( 1 );
		};
		// File offset
		arena_globalIndex[i].offset = offset;
		// To be continued...
//		printf( "Element %d:\t%s\t\t\tOffset: %d\t  Size: %d\n", (i+1), arena_globalIndex[i].name, arena_globalIndex[i].offset, arena_globalIndex[i].size );   // Test
		offset += arena_globalIndex[i].size;
		++i;
	};
//	printf( "GLOBAL.BSA calculated data size: %d\n", (offset-2) );   // Test

	/*  End  */
	fclose( file );
	puts( "" );
	return( 0 );
};


/////////////////////
//  arena_NewData  //
/////////////////////
// Allocate a new data structure
struct arena_data* arena_NewData()
{
	/*  Local Variables  */
	struct arena_data   *data;
	int   slot;

	/*  Look for a free slot  */
	slot = 0;
	while( slot < ARENA_DATA_MAX )   {
		if( arena_dataList[slot] == NULL )   {
			data = arena_dataList[slot];
			break;
		}
		else
			++slot;
	};
	// Not enough slots in data list
	if( slot == ARENA_DATA_MAX )   {
		fprintf( stderr, "arena_NewData error: Not enough slots in Arena data list - Data struct allocation rejected ! Try to free unused data struct or recompile arena.c with an increased ARENA_DATA_MAX value...\n" );
		return( NULL );
	};

	/*  Allocate and clear new data struct  */
	arena_dataList[slot] = (struct arena_data*)calloc( 1, sizeof(struct arena_data) );
	if( arena_dataList[slot] == NULL )   {
		perror( "arena_NewData failed to allocate data struct" );
		return( NULL );
	};

	/*  End  */
	return( arena_dataList[slot] );
};


//////////////////////
//  arena_FreeData  //
//////////////////////
// Free Arena file struct data
void arena_FreeData( struct arena_data *data )
{
	/*  Local Variables  */
	int   i;

	/*  Free content  */
	if( data->data != NULL )
		free( data->data );
	if( data->palette != NULL )
		free( data->palette );

	/*  Find and free data in data list  */
	i = 0;
	while( i < ARENA_DATA_MAX )   {
		if( arena_dataList[i] == data )   {
			free( data );
			arena_dataList[i] = NULL;
			break;
		}
		else
			++i;
	};
	// Data struct isn't owned by this module
	if( i == ARENA_DATA_MAX )
		fprintf( stderr, "arena_FreeData warning: Tried to free an Arena struct owned by another module - Rejected !\n" );
};


/////////////////////
//  arena_FreeAll  //
/////////////////////
/*  Free all Arena allocated data  */
void arena_FreeAll()
{
	/*  Local Variables  */
	int   i;

	/*  Free all data in data list  */
	i = 0;
	while( i < ARENA_DATA_MAX )   {
		if( arena_dataList[i] != NULL )   {
			// Data
			if( arena_dataList[i]->data != NULL )
				free( arena_dataList[i]->data );
			// Palette
			if( arena_dataList[i]->palette != NULL )
				free( arena_dataList[i]->palette );
			// Structure
			free( arena_dataList[i] );
			arena_dataList[i] = NULL;
		}
		++i;
	};

	/*  Free GLOBAL.BSA allocations  */
	// Path
	if( arena_globalPath != NULL )
		free( arena_globalPath );
	// Index
	if( arena_globalIndex != NULL )
		free( arena_globalIndex );
	// Names
	if( arena_globalNames != NULL )
		free( arena_globalNames );
};



/////////////////////////////////////////////////////////////////////////////////////////////
//  END $Source$
/////////////////////////////////////////////////////////////////////////////////////////////
