/////////////////////////////////////////////////////////////////////////////////////////////
//
//		ARENA H     $Revision$
//
/////////////////////////////////////////////////////////////////////////////////////////////

/*  This file was forked from ParseBSA files from Stanza's WinArena open source project  */

/*  By GASNOT Marc ( arkem )
 *
 *  $Date$
 */

/*  Revision History
 *
 *  $Log$
 */



#if !defined _ARENA_H
#define _ARENA_H "@(#)arena.h $Revision$"



/////////////////////////////////////////////////////////////////////////////////////////////
//  PREPROCESSOR
/////////////////////////////////////////////////////////////////////////////////////////////

/*  Misc  */



/////////////////////////////////////////////////////////////////////////////////////////////
//  HEADERS
/////////////////////////////////////////////////////////////////////////////////////////////

/*  Standard  */
#include   <sys/types.h>

/*  Libraries  */

/*  Program  */



/////////////////////////////////////////////////////////////////////////////////////////////
//  DEFINES
/////////////////////////////////////////////////////////////////////////////////////////////

/*  Misc  */
// Files types
enum { ARENA_ANIM=0, ARENA_FONT=1, ARENA_IMAGE=2, ARENA_MAP=3, ARENA_MUSIC=4, ARENA_PALETTE=5, ARENA_SOUND=6, ARENA_TEXT=7, ARENA_LIGHTMAP=8, ARENA_UBIN=9 };



/////////////////////////////////////////////////////////////////////////////////////////////
//  STRUCTURES
/////////////////////////////////////////////////////////////////////////////////////////////

/*  Character for Fonts  */
// Fonts are stored in a packed bit format, one bit per pixel, with each line of a symbol
// packed into 8 or 16 bits, with an array of lines-per-symbol values stored at the beginning
// of the font file.
// Once a font is decoded, it is packed into an array of arena_char structs, so they can be
// easily accessed to mask chars into place by rendering code.
struct arena_char
{
	int         width;
	int         height;
	u_int16_t   lines[16];
};

/*  Color component for Palettes  */
struct arena_color
{
	u_int8_t   red;
	u_int8_t   green;
	u_int8_t   blue;
};

/*  Data  */
struct arena_data
{
	void                 *data;      // Images and raw files: u_int8_t, Fonts: struct arena_char
	struct arena_color   *palette;   // For images
	int                  type;       // ARENA_ANIM, ARENA_FONT, ARENA_IMAGE, ARENA_MAP, ARENA_MUSIC, ARENA_PALETTE, ARENA_SOUND, ARENA_LIGHTMAP, ARENA_TEXT
	int                  length;     // Image number for SET and anims, character number for fonts, size for text and GLOBAL.BSA raw file
	int                  width;      // Width for images and maps
	int                  height;     // Height for images and maps, character Height for fonts
	int                  offsetX;    // X position for some images
	int                  offsetY;    // Y position for some images
};



/////////////////////////////////////////////////////////////////////////////////////////////
//  EXTERNAL VARIABLES
/////////////////////////////////////////////////////////////////////////////////////////////

/*  GLOBAL.BSA Files number  */
extern int   arena_globalNumber;   // Use arena_GlobalList() to obtain a useful value




/////////////////////////////////////////////////////////////////////////////////////////////
//  FONCTIONS DECLARATIONS
/////////////////////////////////////////////////////////////////////////////////////////////

/*  GLOBAL.BSA  */
// Get GLOBAL.BSA files list
extern char** arena_GlobalList( char* );                         // ( GLOBAL.BSA file path ) Return NULL if failed, or a [arena_globalNumber]["filename"] table pointer
// Extract a raw GLOBAL.BSA file
extern struct arena_data* arena_GetGlobalFile( char*, char* );   // ( GLOBAL.BSA file path, file name ) Return NULL if failed
// Replace a raw GLOBAL.BSA file
//extern int arena_PutGlobalFile( struct arena_data*, char* );     // ( data, file name ) Return 0 = OK

/*  Read and write files  */
// Read an Arena format file
extern struct arena_data* arena_ReadFile( char* );         // ( file path ) Return NULL if failed
// Write an Arena format file
extern int arena_WriteFile( struct arena_data*, char* );   // ( data, file path ) Return 0 = OK

/*  Management  */
// Free Arena file struct data
extern void arena_FreeData( struct arena_data* );   // ( data )
// Free all Arena allocated data                    // Required at program exit
extern void arena_FreeAll();



/////////////////////////////////////////////////////////////////////////////////////////////
//  END $Source$
/////////////////////////////////////////////////////////////////////////////////////////////

#endif // _ARENA_H
